#include <vcl.h>
#pragma hdrstop

#include "HTMLCalendarGenerator.h"
#pragma package(smart_init)


__fastcall THTMLCalendarGenerator::THTMLCalendarGenerator(TComponent* Owner) : TComponent(Owner)
   {
   // Initially set date to next month
   unsigned short y, m, d;
   TDateTime date = IncMonth(Date(), 1);
   date.DecodeDate(&y, &m, &d);
   month = m;
   year = y;
   needsGenerating = true;

   onGetEvent = 0;

   output = new TStringList;
   }

__fastcall THTMLCalendarGenerator::~THTMLCalendarGenerator()
   {
   delete output;
   }

void __fastcall THTMLCalendarGenerator::SaveToFile(const String fileName)
   {
   if (needsGenerating) Generate();

   output->SaveToFile(fileName);
   }

void __fastcall THTMLCalendarGenerator::SaveToStream(TStream *stream)
   {
   if (needsGenerating) Generate();

   output->SaveToStream(stream);
   }

void __fastcall THTMLCalendarGenerator::Generate()
   {
   output->Clear();

   TDateTime startDate(year, month, 1);
   TDateTime endDate = IncMonth(startDate, 1) - 1;

   int days = endDate - startDate + 1;
   int startCol = startDate.DayOfWeek() - 1;
   int rowCount = (startCol + days + 6) / 7;
   int cellCount = rowCount * 7;

   // Begin HTML ---------------------------

   output->Add("<html>");

   // HTML Header --------------------------

   output->Add("<head>");
   output->Add("<meta http-equiv='Content-Type' content='text/html'></meta>");
   output->Add("<meta name='GENERATOR' content='HTML Calendar Generator'></meta>");
   output->Add("<title>" + title + FormatDateTime(" - mmmm yyyy", startDate) + "</title>");
   output->Add("</head>");

   // HTML Body -----------------------------

   output->Add("<body>");

   // Calendar Title ------------------------

   output->Add("<table border='1' width='100%' cellspacing='0' cellpadding='5'>");
   output->Add("<tr>");
   output->Add("<td height='80' colspan='7' valign='bottom' align='center'><h2>" + FormatDateTime("mmmm yyyy", startDate) + "</h2>");
   output->Add("<h3>" + title + "</h3></td>");
   output->Add("</tr>");

   // Day of Week Titles ---------------------

   output->Add("<tr>");
   for (int i = 0; i < 7; i++)
   output->Add("<td width='14%' align='center' valign='middle'><font size='2'>" + LongDayNames[i] + "</font></td>");
   output->Add("</tr>");

   // Fill in Calendar ------------------------

   TDateTime date = startDate;
   for (int i = 0; i < cellCount; i++)
      {
      if (i % 7 == 0) output->Add("<tr>");

      if (i >= startCol && i < startCol + days)
         {
         output->Add("<td valign='top' align='left'>");
         output->Add("<font size='2'>" + FormatDateTime("d", date) + "</font><p><font size='1'>");

         String event = "&nbsp;";
         if (onGetEvent) onGetEvent(this, date, event);
         output->Add(event);

         output->Add("</font></td>");

         date++;
         }
      else
         output->Add("<td valign='top' align='left' bgcolor='#EEEEEE'><font size='1'>&nbsp;</font></td>");

      if ((i + 1) % 7 == 0) output->Add("</tr>");
      }

   // End Calendar Table -----------

   output->Add("</table>");

   // End HTML Body ----------------

   output->Add("</body>");

   // End HTML ---------------------

   output->Add("</html>");

   needsGenerating = false;
   }

String __fastcall THTMLCalendarGenerator::GetHTML()
   {
   if (needsGenerating) Generate();

   return(output->Text);
   }

void __fastcall THTMLCalendarGenerator::SetMonth(int m)
   {
   if (m == month) return;

   if (m < 1 || m > 12)
      {
      String msg = String(m)+ " is not a valid month.";
      throw EConvertError(msg);
      }

   month = m;
   needsGenerating = true;
   }

void __fastcall THTMLCalendarGenerator::SetTitle(String t)
   {
   if (title == t) return;

   title = t;
   needsGenerating = true;
   }

void __fastcall THTMLCalendarGenerator::SetYear(int y)
   {
   if (y == year) return;

   if (y < 1900 || y > 3000)
      {
      String msg = String(y)+ " is not a valid year.";
      throw EConvertError(msg);
      }

   year = y;
   needsGenerating = true;
   }


//---------------------------------------------------------------------------

namespace Htmlcalendargenerator
   {
   void __fastcall PACKAGE Register()
      {
      TComponentClass classes[1] = {__classid(THTMLCalendarGenerator)};
      RegisterComponents("BCBJournal", classes, 0);
      }
   }

static inline void ValidCtrCheck(THTMLCalendarGenerator *)
   {
   new THTMLCalendarGenerator(NULL);
   }
